package com.jz.agent.client.service;

import com.google.gson.Gson;
import com.jz.agent.client.DmpDsAgentClient;
import com.jz.agent.utils.JDBCUtils;
import com.jz.common.utils.StringUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DmpMysqlAgentClientService implements DmpDsAgentClient {
	
	private Gson gson = new Gson();
    @SuppressWarnings("unused")
	private static Logger logger = LoggerFactory.getLogger(DmpMysqlAgentClientService.class);

	public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {
		Connection conn = null;
		try {
			Class.forName(ds.getDriverClassName());
			conn = DriverManager.getConnection(
					ds.getJdbcUrl(), 
					ds.getUserName(), 
					ds.getPassword()
			);
			conn.close();
			return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL连通性异常："+e.getMessage());
		}  finally {
			try {
				if(conn != null)
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
		List result = new ArrayList();
		String sql = "select * from information_schema.tables where table_schema = '"+this.getTargetDBname(ds)+"'";
		Connection conn = null;
		ResultSet rs = null;
        PreparedStatement ps = null;
		try {
			conn = JDBCUtils.getConnections(ds.getDriverClassName(), ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				result.add(rs.getString("TABLE_NAME").toLowerCase());
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL获取表列表异常："+e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL获取表列表异常："+e.getMessage());
		} finally {
			try {
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(result));
	}
	
	private String getTargetDBname(DmpAgentDatasourceInfo ds) {
		String url = ds.getJdbcUrl();
		if(url != null && !"".equals(url.trim())) {
			String[] arr1 = url.split("/");
			String ls = arr1[arr1.length-1];
			String[] arr2 = ls.split("\\?");
			return arr2[0];
		} else {
			return "";
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		String dbName = "";
		if (StringUtils.hasLength(ds.getDbName()))
            dbName = " table_schema = '" + ds.getDbName() + "' and ";
        String sql = "select * from information_schema.columns where " + dbName + "table_name = '" + tableName + "'";
		Connection conn = null;
		ResultSet rs;
        PreparedStatement ps;
		try {
			conn = JDBCUtils.getConnections(ds.getDriverClassName(), ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			while (rs.next()) {
				Map map = new HashMap();
				String colName = rs.getString("COLUMN_NAME");
				String dbType = rs.getString("DATA_TYPE");
				String remarks = rs.getString("COLUMN_COMMENT");
				Integer size= 0;
				Integer characterLength = (int) rs.getDouble("CHARACTER_MAXIMUM_LENGTH");
				Integer columnSize = (int) rs.getDouble("NUMERIC_PRECISION");
				Integer dicimalDigits = (int) rs.getDouble("NUMERIC_SCALE");
				if(characterLength != null) {
					size = characterLength;
				} else {
					size = columnSize;
				}
				map.put("name", colName.toLowerCase());
				map.put("type", dbType.toUpperCase());
				map.put("size", size);
				map.put("scale", dicimalDigits);
				map.put("comment", remarks);
				list.add(map);
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL获取表字段信息异常："+e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL获取表字段信息异常："+e.getMessage());
		} finally {
			try {
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
	}

	public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
		Map<String, Object> map = new HashMap<String, Object>();
		Connection conn = null;
		ResultSet rs;
        PreparedStatement ps;
        List<String> headerList = new ArrayList<String>();
		try {
			conn = JDBCUtils.getConnections(ds.getDriverClassName(), ds.getJdbcUrl(), ds.getUserName(), ds.getPassword());
			String sql = "select * from " + tableName + " limit 5";
			String sql2 = "select * from information_schema.columns where table_name = '"+tableName+"' and table_schema = '"+ ds.getDbName() +"' ";
			ps = conn.prepareStatement(sql2);
			rs = ps.executeQuery();
			while (rs.next()) {
				String colName = rs.getString("COLUMN_NAME");
				headerList.add(colName);
			}
			map.put("header", headerList);
			ps = conn.prepareStatement(sql);
			rs = ps.executeQuery();
			List<List<Object>> resultList = new ArrayList<List<Object>>();
            while (rs.next()) {
            	List<Object> row = new ArrayList<Object>();
            	for(int i = 1; i <= headerList.size(); i++) {
            		row.add(rs.getObject(i));
            	}
            	resultList.add(row);
            }
            map.put("result", resultList);
		} catch (SQLException e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL数据预览异常："+e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return new DmpAgentResult(ResultCode.EXCEPTION, "MYSQL数据预览异常："+e.getMessage());
		} finally {
			try {
				conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
	}

}
