package com.jz.dmp.modules.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.agent.service.DmpDsAgentService;
import com.jz.common.constant.CommConstant;
import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import com.jz.common.enums.DelFlagEnum;
import com.jz.common.enums.TaskTreeTypeEnum;
import com.jz.common.page.BasePageBean;
import com.jz.common.page.PageInfoResponse;
import com.jz.common.persistence.BaseService;
import com.jz.common.utils.*;
import com.jz.common.utils.web.SessionUtils;
import com.jz.common.utils.web.XmlUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.modules.controller.DataIntegration.bean.*;
import com.jz.dmp.modules.controller.DataIntegration.bean.flow.FlowExecution;
import com.jz.dmp.modules.dao.*;
import com.jz.dmp.modules.model.*;
import com.jz.dmp.modules.service.DmpDevelopTaskService;
import com.jz.dmp.modules.service.DvTaskRuleTService;
import com.jz.dmp.modules.service.OfflineSynchService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description:离线同步服务层
 * @ClassName: OfflineSynchServiceImpl
 * @Author Bellamy
 * @Date 2020/12/20
 */

@Service("offlineSynchService")
@Transactional
public class OfflineSynchServiceImpl implements OfflineSynchService {

    private static Logger logger = LoggerFactory.getLogger(OfflineSynchServiceImpl.class);

    @Value("${spring.public-key}")
    private String publicKey;

    @Autowired
    private OfflineSynchDao offlineSynchDao;

    @Autowired
    private DmpProjectDao dmpProjectDao;

    @Autowired
    private DmpDevelopTaskDao dmpDevelopTaskDao;

    @Autowired
    private DmpDsAgentService dmpDsAgentServiceImp;

    @Autowired
    private DmpTableDao dmpTableDao;

    @Autowired
    private DmpTableFieldSchemaDao dmpTableFieldSchemaDao;

    @Autowired
    private DmpTableColumnDao dmpTableColumnDao;

    @Autowired
    private DvRuleTDao dvRuleTDao;

    @Autowired
    private DmpNavigationTreeDao dmpNavigationTreeDao;

    @Autowired
    private DmpDevelopTaskService dmpDevelopTaskService;

    @Autowired
    private DvTaskRuleTService dvTaskRuleTService;

    @Autowired
    private DmpDevelopTaskHistoryMapper dmpDevelopTaskHistoryMapper;

    @Autowired
    private DmpRealtimeSyncInfoDao dmpRealtimeSyncInfoDao;

    @Autowired
    private RedisTemplate redisTemplate;


    /**
     * 离线同步任务列表分页查询
     *
     * @return
     * @author Bellamy
     */
    @Override
    public PageInfoResponse<TaskListPageDto> queryTaskListPage(TaskListPageReq taskListPageReq) throws Exception {
        PageInfoResponse<TaskListPageDto> pageInfoResponse = new PageInfoResponse<>();
        if (StringUtils.isNotEmpty(taskListPageReq.getTreeName())) {
            taskListPageReq.setTreeName(taskListPageReq.getTreeName().trim());
        }
        PageHelper.startPage(taskListPageReq.getPageNum(), taskListPageReq.getPageSize());
        List<TaskListPageDto> list = offlineSynchDao.selectOfflineTaskInfo(taskListPageReq);
        PageInfo<TaskListPageDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 获取源数据库名称——下拉框
     */
    @Override
    public JsonResult<List<SourceDbNameListDto>> querygSourceDbList(Integer projectId, String datasourceTypeId) throws Exception {
        Map map = new HashMap();
        map.put("projectId", projectId); //项目id
        map.put("datasourceTypeId", datasourceTypeId);
        List<SourceDbNameListDto> list = offlineSynchDao.querygSourceDbList(map);
        return new JsonResult(ResultCode.SUCCESS, list);
    }

    /**
     * 根据源数据库id,获取源数据表——下拉框
     */
    @Override
    public JsonResult querygSourceTableList(Long sourceDbId, String targetName) throws Exception {
        if (sourceDbId == null) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "数据源id不能为空");
        }
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(sourceDbId);
        if (dsInfo == null) {
            throw new RuntimeException("数据源配置信息不存在！");
        }
        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }
        if (StringUtils.isNotBlank(targetName)) {
            dsInfo.setTargetFileName(targetName);
        }
        //创建jdbc,获取源数据表
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableNameList(dsInfo);
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        } else {
            //成功
            //rst.setResult(JsonMapper.fromJsonString(rst.getMessage(), List.class));
            return new JsonResult(ResultCode.SUCCESS, JsonMapper.fromJsonString(rst.getMessage(), List.class));
        }
    }

    /**
     * 立即运行
     *
     * @author Bellamy
     */
    @Override
    public JsonResult taskRunNowByTaskId(String taskId) throws Exception {
        Map returnMap = new HashMap();
        boolean flag = false;
        //根据任务id，查询DMP资源导航树和DW系统配置信息
        Map<String, Object> map = offlineSynchDao.selectNavigationTreeByTaskId(taskId);
        if (map.size() > 0 && map != null) {
            Integer execId = this.publish(map);
            if (execId != null) {
                flag = true;
                returnMap.put("execId", execId);
                returnMap.put("flag", flag);
            }

            if (flag) {
                return new JsonResult(ResultCode.SUCCESS, returnMap);
            } else {
                return new JsonResult(ResultCode.SYS_ERROR, flag);
            }
        } else {
            return new JsonResult(ResultCode.SYS_ERROR, flag);
        }
    }

    /**
     * 发布流程
     */
    private Integer publish(Map<String, Object> map) throws Exception {
        String taskId = map.get("taskId").toString(); //任务id
        String projectId = map.get("projectId").toString(); //项目id
        String taskType = map.get("type").toString(); //项目id
        String treeName = map.get("treeName").toString(); //任务流程名称
        //String azkabanExectorXmlExec = map.get("azkabanExectorXmlExec").toString(); //执行数据同步任务命令
        String azkabanLocalTaskFilePath = map.get("azkabanLocalTaskFilePath").toString(); //文件路径
        String azkabanMonitorUrl = map.get("azkabanMonitorUrl").toString();//AZKABAN WEB服务地址

        DmpDevelopTask dmpDevelopTask = new DmpDevelopTask();
        String azkabanJobName = dmpDevelopTask.getAzkabanName(taskId);
        String azkabanJobCommand = FlowParseTool.generateExecutorToolCommand(Integer.valueOf(taskId), azkabanJobName, true);

        //String azkabanJobCommand = "command=" + azkabanExectorXmlExec + " " + projectId + " ${azkaban.flow.flowid} ${azkaban.job.id} ${azkaban.flow.execid} " + treeName;

        //校验任务类型并配置参数
        String taskAlias = "";
        if (com.jz.common.utils.StringUtils.isEmpty(taskType)) {
            throw new RuntimeException("任务的任务类型为空！");
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSHELL)) {
            taskAlias = "shell";
        } else if (taskType.equals(CommConstant.TASK_TYPE_DEVSQL)) {
            taskAlias = "sql";
        } else if (taskType.equals(CommConstant.TREE_TYPE_OFFLINE)) {
            taskAlias = "sync";
        } else {
            throw new RuntimeException("该任务类型不能调用此方法发布，请检查任务类型！");
        }

        /**
         * 当前任务生成文件存放根路径
         */
        String localTaskPath = azkabanLocalTaskFilePath + "/" + projectId + "/local/" + taskId;
        File localTaskFile = new File(localTaskPath);
        if (!localTaskFile.exists()) {
            localTaskFile.mkdirs();
        }

        String localTaskSourcePath = localTaskPath + "/source/";
        File localTaskSourceFile = new File(localTaskSourcePath);
        if (!localTaskSourceFile.exists()) {
            localTaskSourceFile.mkdirs();
        }

        String localTaskExecArgsPath = localTaskPath + "/execArgs/";
        File localTaskExecArgsFile = new File(localTaskExecArgsPath);
        if (!localTaskExecArgsFile.exists()) {
            localTaskExecArgsFile.mkdirs();
        }

        String localTaskZipPath = localTaskPath + "/target/";
        File localTaskZipFile = new File(localTaskZipPath);
        if (!localTaskZipFile.exists()) {
            localTaskZipFile.mkdirs();
        }

        //先删除上次创建的文件 ---每次都会新建
        FileUtils.deleteDirectory(localTaskPath);


        List<String> contents = new ArrayList<>();

        // 子流程类型
        contents.add("type=command");
        contents.add(azkabanJobCommand);

        // 生成job文件
        String jobFileAbsolutePath = localTaskSourcePath + azkabanJobName + ".job";
        FileUtils.write(jobFileAbsolutePath, contents);

        String localZipTargetFileName = azkabanJobName + ".zip";
        ZipUtils.zip(localTaskSourcePath, localTaskZipPath, localZipTargetFileName);

        //上传到azkaban todo
        //上次zip包到azkaban
        String localTaskZipAbsolutePath = localTaskZipPath + "/" + localZipTargetFileName;
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl, redisTemplate);
        return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_workflow_new_" + projectId, "local_" + taskAlias + "_project", localTaskZipAbsolutePath, azkabanJobName);
        //return azkabanApiUtils.loginCreateProjectuploadZipAndExecute("jz_localflow_" + projectId, "local_sync_project", localTaskZipAbsolutePath, treeName);
    }

    /**
     * 批量删除离线任务和任务树
     *
     * @author Bellamy
     * @Date 2021/02/03
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult delTaskByTaskId(String taskId) throws Exception {
        //通过taskId，查询任务和资源是否存在
        Map map = dmpDevelopTaskDao.getDmpTaskAndTreeInfo(taskId);
        if (map == null && map.size() == 0) {
            return JsonResult.error(ResultCode.OPERATION_DATA_NO_EXIST);
        }

        Map params = new HashMap();
        String[] ids = taskId.split(",");
        params.put("ids", ids);
        params.put("dataStatus", DelFlagEnum.YES.getValue());
        dmpDevelopTaskDao.deleteTaskByTaskId(params);

        params.put("ids", map.get("treeId").toString());
        dmpNavigationTreeDao.deleteByTreeId(params);
        return JsonResult.ok();
    }

    /**
     * 任务状态查看，异步调用azkaban服务
     *
     * @Author Bellamy
     * @Date 2020/12/20
     */
    @Override
    public JsonResult<List<CheckTaskStatusPageDto>> queryCheckTaskStatusListPage(CheckTaskStatusPageReq checkTaskStatusPageReq) throws Exception {
        List<FlowExecution> list = new ArrayList<>();

        Long projectId = Long.valueOf(checkTaskStatusPageReq.getProjectId()); //项目id
        String treeName = checkTaskStatusPageReq.getTreeName(); //任务树名称
        DmpProjectSystemInfo publishToProjectSystemInfo = dmpProjectDao.queryProjectSystemInfo(projectId);

        //调用azkaban服务
        String azkabanApiUrl = publishToProjectSystemInfo.getAzkabanMonitorUrl();
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanApiUrl, redisTemplate);
        list = azkabanApiUtils.getSyncingFlowExecution(projectId, treeName, checkTaskStatusPageReq.getPageNum(), checkTaskStatusPageReq.getPageSize());

        SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<CheckTaskStatusPageDto> returnList = new ArrayList<>();
        if (list.size() > 0 && list != null) {
            for (FlowExecution str : list) {
                CheckTaskStatusPageDto dto = new CheckTaskStatusPageDto();
                long endTime = str.getEndTime();
                long startTime = str.getStartTime();
                dto.setCostTime(endTime - startTime); //耗时
                dto.setExecId(str.getExecId()); //任务执行id
                dto.setStatus(str.getStatus());
                dto.setSubmitUser(str.getSubmitUser());
                dto.setEndTime(dtf.format(new Date(endTime))); //结束时间
                dto.setStartTime(dtf.format(new Date(startTime))); //开始时间
                returnList.add(dto);
            }
        }
        return new JsonResult(ResultCode.SUCCESS, returnList);
    }

    @Override
    public PageInfoResponse<CheckJyRlueStatusDto> selectCheckJyStatusInfo(CheckJyRlueStatusReq checkJyRlueStatusReq) throws Exception {
        PageInfoResponse<CheckJyRlueStatusDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(checkJyRlueStatusReq.getPageNum(), checkJyRlueStatusReq.getPageSize());
        List<CheckJyRlueStatusDto> list = offlineSynchDao.selectCheckJyStatusInfo(checkJyRlueStatusReq.getExecutionId());
        PageInfo<CheckJyRlueStatusDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 获取源表和目标表的字段
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult querySoureAndTargetColumnsByParams(SoureAndTargetColumnsReq soureAndTargetColumnsReq) throws Exception {
        //通过源数据库id ，查询数据源配置
        DmpAgentDatasourceInfo dsInfo = offlineSynchDao.querySourceDbInfoBySourceId(soureAndTargetColumnsReq.getSourceDbId());

        //通过目标数据库id ，查询数据配置
        DmpAgentDatasourceInfo targetInfo = offlineSynchDao.querySourceDbInfoBySourceId(soureAndTargetColumnsReq.getTargetDbId());

        //dsInfo.setHdfsUserName(body.getAccessId()); // 专为HDFS CLIENT提供
        dsInfo.setFileType(soureAndTargetColumnsReq.getFileType());
        dsInfo.setDelimiter(soureAndTargetColumnsReq.getCsvDelimiter());
        dsInfo.setIsHaveHeader(soureAndTargetColumnsReq.getCsvIsHaveHeader());

        //解码源数据库密码
        if (StringUtils.isNotBlank(dsInfo.getPassword())) {
            dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
        }

        //创建jdbc,获取源数据表字段
        DmpAgentResult rst = dmpDsAgentServiceImp.getTableColumnList(dsInfo, soureAndTargetColumnsReq.getTargetTableName());
        if (!rst.getCode().val().equals("200")) {
            return new JsonResult(rst.getCode(), rst.getMessage());
        }
        List<Map<String, Object>> cols = (List<Map<String, Object>>) JsonMapper.fromJsonString(rst.getMessage(), List.class);
        if (cols == null || cols.size() == 0) {
            return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR, "获取不到对应数据源的数据");
        }
        logger.info("######################################  数据源的表字段 success  ####################################");

        DmpTable table = new DmpTable(); //项目表信息
        table.setTableMode(dsInfo.getDatasourceTypeId()); //数据源类型ID
        table.setCnName(soureAndTargetColumnsReq.getTargetTableName()); //中文名
        table.setIsShow("0"); //是否隐藏
        table.setName(soureAndTargetColumnsReq.getTargetTableName());//表名
        table.setTableDesc("数据同步-源表"); //描述
        table.setDataStatus("1"); //数据表状态 默认1

        dmpTableDao.deleteTableColumnByName(table);
        dmpTableDao.deleteTableByName(table);
        dmpTableDao.insert(table);

        DmpTableFieldSchema schema = new DmpTableFieldSchema(); //项目表字段类型
        schema.setDstId(Integer.valueOf(dsInfo.getDatasourceTypeId()));
        List<DmpTableFieldSchema> sourceFieldSchema = dmpTableFieldSchemaDao.queryAll(schema); //源字段架构
        schema.setDstId(Integer.valueOf(dsInfo.getDatasourceTypeId()));
        List<DmpTableFieldSchema> targetFieldSchema = dmpTableFieldSchemaDao.queryAll(schema); //目标字段架构
        if (sourceFieldSchema != null && sourceFieldSchema.size() > 0) {
            Map<String, Integer> ftm = new HashMap<String, Integer>();
            for (DmpTableFieldSchema s : sourceFieldSchema) { //便利源字段类型
                ftm.put(s.getFieldType(), s.getId().intValue());
            }
            List<DmpTableColumn> columns = new ArrayList<DmpTableColumn>();
            for (int i = 0; i < cols.size(); i++) { //便利源字段
                Map<String, Object> map = cols.get(i);
                String name = (String) map.get("name");
                String type = (String) map.get("type");
                Integer size = (Integer) map.get("size");
                Integer scale = (Integer) map.get("scale");
                String comment = (String) map.get("comment");

                DmpTableColumn dtc = new DmpTableColumn();
                dtc.setField(name); //字段
                dtc.setType(ftm.get(type)); //类型
                if (size != null) dtc.setSize(size); //大小
                if (scale != null) dtc.setScale(scale); //经度
                dtc.setTableId(table.getId().intValue()); //tableID
                dtc.setComment(comment); //备注说明
                dtc.setDataStatus("1"); //数据表状态 默认1
                columns.add(dtc);
            }
            //向dmp_table_column批量新增
            dmpTableColumnDao.insertBatch(columns);
        }

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("targetDsId", targetInfo.getDatasourceTypeId()); //数据源类型ID
        params.put("sourceDsId", dsInfo.getDatasourceTypeId());
        params.put("tableName", soureAndTargetColumnsReq.getTargetTableName());
        List<Map<String, Object>> fieldsMappingList = dmpTableFieldSchemaDao.getTableFieldsMapping(params);
        List<Map<String, Object>> sourceFieldColumns = new ArrayList<Map<String, Object>>();
        List<Map<String, Object>> targetFieldColumns = new ArrayList<Map<String, Object>>();
        for (Map<String, Object> m : fieldsMappingList) {
            String field = (String) m.get("field");
            String sourceType = (String) m.get("sourceType");
            String targetType = "STRING";
            if (m.get("targetType") != null && StringUtils.isNotBlank((String) m.get("targetType"))) {
                targetType = (String) m.get("targetType");
            }
            Integer size = (Integer) m.get("size");
            Integer scale = (Integer) m.get("scale");

            Map<String, Object> sourceFieldColumn = new HashMap<String, Object>();//源表字段
            sourceFieldColumn.put("name", field); //字段
            sourceFieldColumn.put("type", sourceType); //类型
            sourceFieldColumn.put("size", size); //大小
            sourceFieldColumn.put("scale", scale); //经度

            Map<String, Object> targetFieldColumn = new HashMap<String, Object>();//目标表字段
            targetFieldColumn.put("name", field);
            targetFieldColumn.put("type", targetType);
            targetFieldColumn.put("size", size);
            targetFieldColumn.put("scale", scale);

            sourceFieldColumns.add(sourceFieldColumn);
            targetFieldColumns.add(targetFieldColumn);
        }
        Map<String, Object> bak = new HashMap<String, Object>();
        //bak.put("sourceFieldSchema", sourceFieldSchema);
        bak.put("sourceFieldColumns", sourceFieldColumns);
        //bak.put("targetFieldSchema", targetFieldSchema);
        bak.put("targetFieldColumns", targetFieldColumns);

        dmpTableDao.deleteById(table.getId());
        dmpTableColumnDao.deleteByTableId(table.getId());
        return new JsonResult(ResultCode.SUCCESS, bak);
    }

    @Override
    public PageInfoResponse<DvRuleTDto> queryJyRuleListPage(BasePageBean basePageBean, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DvRuleTDto> pageInfoResponse = new PageInfoResponse<>();

        PageHelper.startPage(basePageBean.getPageNum(), basePageBean.getPageSize());
        DvRuleT dvRuleT = new DvRuleT();
        List<DvRuleTDto> list = dvRuleTDao.queryJyRuleListPage(dvRuleT);
        if (list.size() > 0 && list != null) {
            for (DvRuleTDto dto : list) {
                dto.setRuleContentJson(JSON.parseObject(dto.getRuleContent()));
            }
        }
        PageInfo<DvRuleTDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(ResultCode.SUCCESS);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }

    /**
     * 添加保存dmp数据(包含校验数据)
     *
     * @param syncDmpTaskAddReq
     * @return
     * @Author Bellamy
     * @Date 2020/12/20
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult addSyncTask(SyncDmpTaskAddReq syncDmpTaskAddReq) throws Exception {
        Map<String, Object> reqParam = syncDmpTaskAddReq.getParams();
        if (reqParam.size() == 0 || reqParam == null) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "请求参数不能为空");
        }
        JsonResult jsonResult = addSyncing(reqParam);
        return jsonResult;
    }

    /*
     *编辑离线任务数据
     * @Author Bellamy
     * @Date 2020/12/30
     * */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult updateSyncTask(SyncDmpTaskAddReq syncDmpTaskAddReq) throws Exception {
        Map<String, Object> reqParam = syncDmpTaskAddReq.getParams();
        if (reqParam.size() > 0 && reqParam != null) {
            Integer projectId = Integer.valueOf(reqParam.get("projectId").toString());
            Integer parentId = Integer.valueOf(reqParam.get("parentId").toString()); //父节点ID
            String taskName = (String) reqParam.get("taskName"); //任务名称 业务节点名称 一对一
            Integer treeId = Integer.valueOf(reqParam.get("treeId").toString()); //树节点ID
            Integer taskId = Integer.valueOf(reqParam.get("taskId").toString()); //任务ID

            if (StringUtils.isBlank(taskName)) {
                return new JsonResult(ResultCode.PARAMS_ERROR, "任务名称不能为空");
            }
            Map<String, Object> scriptMap = (Map<String, Object>) reqParam.get("scripts"); //任务json数据
            Object content = scriptMap.get("content");
            String xmlTdb = XmlUtils.getPropertyValue(content, "target_db_connection");
            if (null != content && xmlTdb.length() == 0) { // 包含content但未取出值条件才成立
                return new JsonResult(ResultCode.PARAMS_ERROR, "脚本内容中缺失目标数据源(target_db_connection)");
            }

            //保存目标库类型
            Map<String, Object> writer = (Map<String, Object>) scriptMap.get("writer");  // 目标数据
            String targetDb = (String) writer.get("targetDbConnection"); // 目标库名称
            String targetTable = (String) writer.get("targetTable"); // 目标库表名称

            Map<String, Object> reader = (Map<String, Object>) scriptMap.get("reader");//源数据
            String sourceDbName = (String) reader.get("dbConnection"); // 源库名称
            String sourceTableName = (String) reader.get("registerTableName"); // 源库表名称

            Integer dataSourceId = null; //数据源ID
            if (StringUtils.isNotBlank(targetDb)) {
                //根据 目标库和项目id 查询
                dataSourceId = dmpDevelopTaskDao.getDbInfoByParam(targetDb, projectId);
            }

            DmpDevelopTask task = new DmpDevelopTask();
            task.setProjectId(projectId);
            task.setParentId(parentId);
            task.setTaskType("2"); //任务类型
            task.setDatasourceId(dataSourceId); //数据源ID
            task.setType("3");
            //task.setTaskName(taskName);
            task.setTaskDesc("Syncing Task"); //任务描述
            task.setIsSubmit("0"); //是否已提交
            task.setTreeId(treeId);
            task.setDataStatus("1");
            String script = JsonMapper.toJsonString(reqParam);
            byte[] data = null;
            try {
                data = script.getBytes("utf-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            task.setData(data); //json 数据
            task.setTargetDbName(targetDb);
            task.setTargetTableName(targetTable);
            task.setSourceTableName(sourceTableName);
            task.setSourceDbName(sourceDbName);
            task.setId(taskId);

            dmpDevelopTaskDao.update(task); //更新任务
            logger.info("======== edit sync task end ========");

            //更新规则信息
            List<DvTaskRuleT> list = new ArrayList<>();
            //查询TaskRuleID 集合
            List<Long> listRules = dvTaskRuleTService.getTaskRuleIdsList(taskId);
            if (CollectionUtils.isNotEmpty(listRules)) {
                //批量删除rule信息
                dvTaskRuleTService.delRulesByIds(listRules);
                List<Map> taskRules = (List<Map>) reqParam.get("taskRules");
                //保存dmp数据校验规则信息
                settRuleInfo(String.valueOf(taskId), taskRules, list);
            }
            //保存时提交XML
            dmpDevelopTaskService.submitSyncing(task);
        }
        return new JsonResult(ResultCode.SUCCESS);
    }

    /**
     * 添加保存离线任务dmp数据
     *
     * @param body
     * @return
     */
    public JsonResult addSyncing(Map<String, Object> body) throws Exception {
        if (StringUtils.isEmpty(String.valueOf(body.get("treeId")))) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "treeId不能为空！");
        }
        if (StringUtils.isEmpty(body.get("projectId").toString())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "projectId不能为空！");
        }
        if (StringUtils.isBlank(body.get("taskName").toString())) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "任务名称不能为空");
        }

        Integer projectId = Integer.valueOf(body.get("projectId").toString());
        String taskId = body.get("taskId").toString(); //任务ID
        Integer treeId = Integer.valueOf(body.get("treeId").toString()); //树节点ID

        Map<String, Object> scriptMap = (Map<String, Object>) body.get("scripts"); //任务json数据
        Object content = scriptMap.get("content");
        String xmlTdb = XmlUtils.getPropertyValue(content, "target_db_connection");
        if (null != content && xmlTdb.length() == 0) { // 包含content但未取出值条件才成立
            return JsonResult.error(ResultCode.PARAMS_ERROR, "脚本内容中缺失目标数据源(target_db_connection)");
        }

        //保存目标库类型
        Map<String, Object> writer = (Map<String, Object>) scriptMap.get("writer");  // 目标数据
        String targetDb = String.valueOf(writer.get("targetDbConnection")); // 目标库名称
        String targetDbId = String.valueOf(writer.get("targetDbId")); // 目标库id
        String targetTable = (String) writer.get("targetTable"); // 目标库表名称
        if (StringUtils.isEmpty(targetDbId)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "targetDbId不能为空！");
        }
        //源数据
        Map<String, Object> reader = (Map<String, Object>) scriptMap.get("reader");
        String sourceDbName = (String) reader.get("dbConnection"); // 源库名称
        String sourceDbId = (String) reader.get("sourceDbId"); // 来源库id
        String sourceTableName = (String) reader.get("registerTableName"); // 源库表名称
        if (StringUtils.isEmpty(sourceDbId)) {
            return JsonResult.error(ResultCode.PARAMS_ERROR, "sourceDbId不能为空！");
        }

        DmpDevelopTask task = new DmpDevelopTask();
        task.setProjectId(projectId);
        task.setTaskType("2"); //任务类型
        task.setType("3");
        task.setTaskDesc("Syncing Task"); //任务描述
        task.setIsSubmit("0"); //是否已提交
        task.setTreeId(treeId);
        task.setDataStatus("1");
      /*  String script = JsonMapper.toJsonString(body);
        byte[] data = null;
        try {
            data = script.getBytes("utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        task.setData(data); //json 数据*/
        task.setScript(JSONObject.toJSONString(body));
        task.setTargetDbName(targetDb);
        task.setTargetTableName(targetTable);
        task.setSourceTableName(sourceTableName);
        task.setSourceDbName(sourceDbName);
        task.setSourceDbId(sourceDbId);
        List<DvTaskRuleT> list = new ArrayList<>();
        //更新规则信息
        List<Map> taskRules = (List<Map>) body.get("taskRules");
        //判断taskId是否存在，存在编辑，不存在新增
        Integer id = null;
        if (StringUtils.isEmpty(taskId)) {
            task.setVersion("1.0");
            task.setCreateUserId(SessionUtils.getCurrentUserId());
            task.setCreateTime(new Date());
            dmpDevelopTaskDao.insert(task); //新增任务数据
            id = task.getId();
            logger.info("======== save sync task end ========");
            this.saveTaskHistory(task); //保存任务历史版本
            //保存dmp数据校验规则信息
            settRuleInfo(task.getId().toString(), taskRules, list);
        } else {
            DmpDevelopTask devTask = dmpDevelopTaskDao.selectTaskById(taskId);
            BigDecimal version = new BigDecimal(devTask.getVersion());
            version = version.add(new BigDecimal(1.0));
            task.setVersion(String.valueOf(version));
            task.setUpdateTime(new Date());
            task.setUpdateUserId(SessionUtils.getCurrentUserId());
            task.setId(Integer.valueOf(taskId));
            dmpDevelopTaskDao.update(task);
            id = task.getId();
            logger.info("======== edit sync task end ========");
            this.saveTaskHistory(task); //保存任务历史版本
            //更新规则信息
            //查询TaskRuleID 集合
            List<Long> listRules = dvTaskRuleTService.getTaskRuleIdsList(Integer.valueOf(taskId));
            if (CollectionUtils.isNotEmpty(listRules)) {
                //批量删除rule信息
                dvTaskRuleTService.delRulesByIds(listRules);
                //保存dmp数据校验规则信息
                settRuleInfo(String.valueOf(taskId), taskRules, list);
            }
        }
        //保存时提交XML
        //dmpDevelopTaskService.submitSyncing(task);
        Map map = new HashMap();
        map.put("id", id);
        return JsonResult.ok(map);
    }

    /**
     * 保存离线任务历史版本
     *
     * @param task
     * @return
     * @Author Bellamy
     * @Date 2021/02/01
     */
    public JsonResult saveTaskHistory(DmpDevelopTask task) throws Exception {
        if (task.getId() == null) {
            throw new RuntimeException("保存失败！");
        }
        DmpDevelopTaskHistory taskHistory = new DmpDevelopTaskHistory();
        taskHistory.setTaskId(task.getId());
        taskHistory.setVersion(task.getVersion());
        taskHistory.setScript(task.getScript());
        taskHistory.setTaskDesc(task.getTaskDesc());
        taskHistory.setTreeId(task.getTreeId());
        taskHistory.setTaskType(task.getTaskType());
        taskHistory.setDatasourceId(task.getDatasourceId());
        taskHistory.setCreateTime(new Date());
        taskHistory.setCreateUserId(Integer.valueOf(SessionUtils.getCurrentUserId()));
        taskHistory.setDataStatus(DelFlagEnum.NO.getValue());
        dmpDevelopTaskHistoryMapper.insert(taskHistory);
        logger.info("======== save sync task history end ========");
        return JsonResult.ok();
    }

    /**
     * 保存规则信息
     *
     * @param result
     * @param reqParam
     * @param jsonResult
     * @param taskId
     * @throws Exception
     */
    private void saveRuleInfo(List<Map<String, Object>> result, Map<String, Object> reqParam, JsonResult jsonResult, String taskId) throws Exception {
        List<DvTaskRuleT> list = new ArrayList<>();
        if (jsonResult.getCode().equals(ResultCode.SUCCESS.val())) {
            List<Map> rules = (List<Map>) reqParam.get("taskRules"); //规则信息
            if (!CollectionUtils.isEmpty(rules)) {
                settRuleInfo(taskId, rules, list);
            }
        } else {
            HashMap<String, Object> errorMap = new HashMap<>();
            errorMap.put("name", reqParam.get("name"));
            errorMap.put("msg", jsonResult.getMessage());
            errorMap.put("code", jsonResult.getCode());
            result.add(errorMap);
        }
    }

    /**
     * 保存dmp数据校验规则信息
     *
     * @param taskId
     * @param rules
     * @param list
     * @throws Exception
     */
    private void settRuleInfo(String taskId, List<Map> rules, List<DvTaskRuleT> list) throws Exception {
        if (rules.size() > 0 && rules != null) {
            for (Map rule : rules) {
                if (StringUtils.isNotEmpty(rule.get("ruleId").toString())) {
                    DvTaskRuleT taskRuleT = new DvTaskRuleT();
                    String ruleValue = (String) rule.get("ruleValue");
                    taskRuleT.setTaskId(taskId);//任务ID
                    taskRuleT.setRuleId(Long.valueOf(rule.get("ruleId").toString()));
                    taskRuleT.setRuleValue(ruleValue);
                    list.add(taskRuleT);
                }
            }
            //批量新增任务与规则关系表
            if (list.size() > 0 && list != null)
                dvTaskRuleTService.saveRule(list);
        }
    }

    /**
     * 获取数据源表字段
     *
     * @return
     * @author Bellamy
     * @since 2021-01-21
     */
    @Override
    public JsonResult querySoureTableColumns(Map<String, List<SynchTableColumnsReq>> req) throws Exception {
        List<Map> returnData = new ArrayList<>();
        int len = 0;
        if (req.size() == 0 || req == null) {
            throw new RuntimeException("请求参数不能为空！");
        }
        List<SynchTableColumnsReq> list = req.get("params");
        for (SynchTableColumnsReq str : list) {
            if (null != str.getSourceDbId() && null != str.getTargetTableName()) {
                //通过源数据库id ，查询数据源配置
                DmpAgentDatasourceInfo dsInfos = offlineSynchDao.querySourceDbInfoBySourceId(str.getSourceDbId());
                DmpAgentDatasourceInfo dsInfo = new DmpAgentDatasourceInfo();
                BeanUtils.copyProperties(dsInfos, dsInfo);
                if (dsInfo == null) {
                    throw new RuntimeException("数据源配置信息不存在！");
                }
                //解码源数据库密码
                if (StringUtils.isNotBlank(dsInfo.getPassword())) {
                    dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
                }

                //创建jdbc,获取数据源表字段
                DmpAgentResult rst = dmpDsAgentServiceImp.getTableColumnList(dsInfo, str.getTargetTableName());
                if (!rst.getCode().val().equals("200")) {
                    return new JsonResult(rst.getCode(), rst.getMessage());
                } else {
                    //成功
                    List<Map> returnList = (List<Map>) JsonMapper.fromJsonString(rst.getMessage(), List.class);
                    if (returnList != null && returnList.size() > 0) {
                        for (int i = 0; i < returnList.size(); i++) {
                            Map map = returnList.get(i);
                            //离线同步
                            map.put("id", ++len);
                            map.put("fieldAlias", map.get("name")); //字段别名
                            map.put("isPk", 0); //主键
                            map.put("isPt", 0); //分区
                            //实时同步：脱敏算法
                            map.put("arithmetic", "HmacSHA256");
                            returnData.add(map);
                        }
                    }
                }
            }
        }

        if (StringUtils.isNotEmpty(list.get(0).getFieldName())) {
            List<Map> searchFieldList = getFieldName(returnData, list.get(0).getFieldName());
            return JsonResult.ok(searchFieldList);
        }
        return JsonResult.ok(returnData);
    }

    /**
     * 根据参数搜索表字段
     *
     * @return
     * @author Bellamy
     * @since 2021-03-02
     */
    public List<Map> getFieldName(List<Map> list, String fieldName) throws Exception {
        List<Map> returnList = new ArrayList<>();
        if (list.size() > 0 && null != list) {
            for (Map str : list) {
                String name = (String) str.get("name");
                if (name.contains(fieldName)) {
                    returnList.add(str);
                }
            }
        }
        return returnList;
    }

    /**
     * 任务停止运行
     *
     * @return
     * @author Bellamy
     * @since 2021-01-07
     */
    @Override
    public JsonResult stopSyncingTaskByTaskId(String taskId) throws Exception {
        //根据任务id，查询DMP资源导航树和DW系统配置信息
        Map<String, Object> map = offlineSynchDao.selectNavigationTreeByTaskId(taskId);
        if (map == null || map.size() == 0) {
            throw new RuntimeException("任务数据信息不存在");
        }
        String azkabanMonitorUrl = map.get("azkabanMonitorUrl").toString();
        Long projectId = Long.valueOf(map.get("projectId").toString());
        AzkabanApiUtils2 azkabanApiUtils = new AzkabanApiUtils2(azkabanMonitorUrl, redisTemplate);
        String execId = azkabanApiUtils.stopFlow("jz_localflow_" + projectId, map.get("treeName").toString());
        Map returnMap = new HashMap();
        returnMap.put("execId", execId);
        return JsonResult.ok(returnMap);
    }

    /**
     * 新增同步任务
     *
     * @return
     * @author Bellamy
     * @since 2021-01-26
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public JsonResult addNewSynchTask(NewSynchTaskReq newSynchTaskReq) throws Exception {
        DmpNavigationTree dmpNavigationTree = new DmpNavigationTree();
        dmpNavigationTree.setName(newSynchTaskReq.getTreeName());
        dmpNavigationTree.setDataStatus("1");
        List<DmpNavigationTree> list = dmpNavigationTreeDao.queryAll(dmpNavigationTree);
        if (list.size() > 0 && list != null) {
            return JsonResult.error("任务名称已存在！");
        }
        DmpNavigationTree tree = new DmpNavigationTree();
        tree.setCreateTime(new Date());
        tree.setCreateUserId(SessionUtils.getCurrentUserId());
        tree.setName(newSynchTaskReq.getTreeName());
        tree.setProjectId(Integer.valueOf(newSynchTaskReq.getProjectId()));
        tree.setType(newSynchTaskReq.getType());
        tree.setParentId(Integer.valueOf(newSynchTaskReq.getParentId()));
        tree.setIsLevel("1");
        tree.setIsEnable("1");
        tree.setDataStatus("1");
        int len = dmpNavigationTreeDao.insert(tree);
        if (len > 0) {
            if (TaskTreeTypeEnum.SYNC.getValue().equals(newSynchTaskReq.getType())) {
                DmpDevelopTask dmpDevelopTask = new DmpDevelopTask();
                dmpDevelopTask.setTreeId(tree.getId());
                dmpDevelopTask.setVersion("1.0");
                dmpDevelopTask.setCreateTime(new Date());
                dmpDevelopTask.setCreateUserId(SessionUtils.getCurrentUserId());
                dmpDevelopTask.setDataStatus(DelFlagEnum.NO.getValue());
                dmpDevelopTask.setName(tree.getName());
                dmpDevelopTask.setTaskType("2");
                dmpDevelopTaskDao.insert(dmpDevelopTask);
                return JsonResult.ok(dmpDevelopTask);
            } else if (TaskTreeTypeEnum.SSTB.getValue().equals(newSynchTaskReq.getType())) {
                DmpRealtimeSyncInfo dmpRealtimeSyncInfo = new DmpRealtimeSyncInfo();
                dmpRealtimeSyncInfo.setTreeId(String.valueOf(tree.getId()));
                dmpRealtimeSyncInfo.setVersion("1.0");
                dmpRealtimeSyncInfo.setCrePerson(SessionUtils.getCurrentUserId());
                dmpRealtimeSyncInfo.setCreateTime(new Date());
                dmpRealtimeSyncInfo.setProjectId(newSynchTaskReq.getProjectId());
                dmpRealtimeSyncInfo.setName(tree.getName());
                dmpRealtimeSyncInfoDao.insert(dmpRealtimeSyncInfo);
                return JsonResult.ok(dmpRealtimeSyncInfo);
            }
        }
        return JsonResult.error();
    }

    /**
     * 数据预览
     *
     * @param req
     * @return
     * @author Bellamy
     * @since 2021-02-19
     */
    @Override
    public JsonResult getDataPreview(OfflineDataPreview req) throws Exception {
        List<Object> listHeard = new ArrayList<>();
        List<Map> tableData = new ArrayList<>();
        Map returnData = new HashMap();
        if (req.getParams() == null || req.getParams().size() == 0) {
            throw new RuntimeException("请求参数不能为空！");
        }
        for (int m = 0; m < req.getParams().size(); m++) {
            SynchTableColumnsReq str = req.getParams().get(m);
            //通过源数据库id ，查询数据源配置
            DmpAgentDatasourceInfo dsInfos = offlineSynchDao.querySourceDbInfoBySourceId(str.getSourceDbId());
            DmpAgentDatasourceInfo dsInfo = new DmpAgentDatasourceInfo();
            BeanUtils.copyProperties(dsInfos, dsInfo);
            if (dsInfo == null) {
                throw new RuntimeException("数据源配置信息不存在！");
            }
            //解码源数据库密码
            if (StringUtils.isNotBlank(dsInfo.getPassword())) {
                dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
            }
            //只展示第一张表数据
            if (m > 0) {
                break;
            }
            //创建jdbc,获取数据源数据预览
            DmpAgentResult rst = dmpDsAgentServiceImp.previewData(dsInfo, str.getTargetTableName());
            if (!rst.getCode().val().equals("200")) {
                return new JsonResult(rst.getCode(), rst.getMessage());
            } else {
                //成功,包含表字段和行数据
                Map<String, List<Object>> returnList = (Map<String, List<Object>>) JSONObject.parse(rst.getMessage());

                if (returnList != null && returnList.size() > 0) {
                    List<Object> header = returnList.get("header"); //表头 字段
                    listHeard.addAll(header); //拼接每张表头

                    List<List<Object>> columnData = Collections.singletonList(returnList.get("result"));//行数据
                    List<Object> rowData = columnData.get(0);
                    for (int i = 0; i < rowData.size(); i++) {//表所有数据
                        List<Object> row = (List<Object>) rowData.get(i); //每行数据
                        Map colKeyValue = new LinkedHashMap();
                        for (int j = 0; j < header.size(); j++) { //表头
                            for (int h = 0; h < row.size(); h++) {
                                if (j == h) {
                                    System.out.print(header.get(j) + " " + row.get(h));
                                    colKeyValue.put(header.get(j), row.get(h));
                                    break;
                                }
                            }
                        }
                        tableData.add(colKeyValue);
                    }
                }

               /* if (returnList != null && returnList.size() > 0) {
                    List<Object> header = returnList.get("header"); //表头 字段
                    listHeard.addAll(header); //拼接每张表头
                    List<List<Object>> columnData = Collections.singletonList(returnList.get("result"));//行数据
                    for (int i = 0; i < columnData.size(); i++) {
                        List<Object> rowData = columnData.get(i);
                        Map<String, List<Object>> map1 = new HashMap<>();
                        map1.put("value", rowData);
                        data1.add(map1);
                    }
                }*/
            }
        }
       /* //两张表字段横向拼接展示 ，data1所有表的数据
        if (data1.size() > 0 && data1 != null) {
            for (int i = 0; i < data1.size(); i++) {
                Map mapa = data1.get(i);
                List<List<Object>> lista = (List<List<Object>>) mapa.get("value"); //遍历第i个表数据
                List<List<Object>> reData = new ArrayList<>(); //每次循环将数据添加进去

                int c = ++i;
                for (int a = 0; a < lista.size(); a++) {//遍历第i个表数据
                    List<Object> obj1 = lista.get(a);
                    if (c < data1.size()) {
                        Map mapb = data1.get(c);
                        List<List<Object>> listb = (List<List<Object>>) mapb.get("value");
                        for (int b = 0; b < listb.size(); b++) { //遍历第i+1个表数据
                            List<Object> obj2 = listb.get(b);
                            if (a == b) {
                                obj1.addAll(obj2);
                                break;
                            }
                        }
                    }
                    reData.add(obj1);
                }
                mapa.put("value", reData);
            }
        }
*/
        returnData.put("header", listHeard);
        returnData.put("rowDatas", tableData);
        return JsonResult.ok(returnData);
    }

    /**
     * sync-获取数据源表字段
     *
     * @param req
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult getSyncSoureAndTargetColumns(Map<String, List<SynchTableColumnsReq>> req) throws Exception {
        List<Map> returnData = new ArrayList<>();
        List<Map> syncData = new ArrayList<>();
        int len = 0;
        if (req.size() == 0 || req == null) {
            throw new RuntimeException("请求参数不能为空！");
        }
        List<SynchTableColumnsReq> list = req.get("params");
        Long id = list.get(0).getSourceDbId();
        int num = 0;
        for (SynchTableColumnsReq str : list) {
            num++;
            if (null != str.getSourceDbId() && null != str.getTargetTableName()) {
                //通过源数据库id ，查询数据源配置
                DmpAgentDatasourceInfo dsInfos = offlineSynchDao.querySourceDbInfoBySourceId(str.getSourceDbId());
                DmpAgentDatasourceInfo dsInfo = new DmpAgentDatasourceInfo();
                BeanUtils.copyProperties(dsInfos, dsInfo);
                if (dsInfo == null) {
                    throw new RuntimeException("数据源配置信息不存在！");
                }
                //解码源数据库密码
                if (StringUtils.isNotBlank(dsInfo.getPassword())) {
                    dsInfo.setPassword(new BaseService().decode(dsInfo.getPassword(), publicKey));
                }

                //创建jdbc,获取数据源表字段
                DmpAgentResult rst = dmpDsAgentServiceImp.getTableColumnList(dsInfo, str.getTargetTableName());
                if (!rst.getCode().val().equals("200")) {
                    return new JsonResult(rst.getCode(), rst.getMessage());
                } else {
                    //成功
                    List<Map> returnList = (List<Map>) JsonMapper.fromJsonString(rst.getMessage(), List.class);
                    if (returnList != null && returnList.size() > 0) {
                        for (int i = 0; i < returnList.size(); i++) {
                            Map map = returnList.get(i);
                            //离线同步
                            map.put("id", ++len);
                            map.put("fieldAlias", map.get("name")); //字段别名
                            map.put("isPk", 0); //主键
                            map.put("isPt", 0); //分区
                            returnData.add(map);
                            if (num == 1) {
                                syncData.add(map);
                            }
                        }
                    }
                }
            }
        }
        Map data = new HashMap();
        data.put("allTable", returnData);
        data.put("firstTable", syncData);
        return JsonResult.ok(data);
    }

    /**
     * 离线任务详情--编辑回显
     *
     * @return
     * @author Bellamy
     */
    @Override
    public JsonResult querySyncTaskInfoById(String id) throws Exception {
        Map returnMap = new HashMap();
        Map sourceMap = new HashMap();
        Map targetMap = new HashMap();
        DmpDevelopTask dmpDevelopTask = dmpDevelopTaskDao.selectTaskById(id);
        if (StringUtils.isNotEmpty(dmpDevelopTask.getScript())) {
            Map map = (Map) JSONObject.parse(dmpDevelopTask.getScript());
            Map scripts = (Map) map.get("scripts");
            Integer projectId = Integer.valueOf((String) map.get("projectId"));
            Map reader = (Map) scripts.get("reader");
            Map writer = (Map) scripts.get("writer");
            String sourceDbType = String.valueOf(reader.get("sourceDbType"));
            String sourceDbId = String.valueOf(reader.get("sourceDbId"));
            String[] sourceDbName = String.valueOf(reader.get("dbConnection")).split(",");
            String targetDbType = String.valueOf(writer.get("targetDbType"));
            String targetDbId = String.valueOf(writer.get("targetDbId"));
            String targetName = String.valueOf(writer.get("targetDbConnection"));

            //数据源
            JsonResult sourceDb = querygSourceDbList(projectId, sourceDbType);
            sourceMap.put("sourceDb", sourceDb.getData());
            JsonResult targetDb = querygSourceDbList(projectId, targetDbType);
            targetMap.put("targetDb", targetDb.getData());

            //表
            String[] sourceDbIds = sourceDbId.split(",");
            for (int i = 0; i < sourceDbIds.length; i++) {
                JsonResult sourceTable = querygSourceTableList(Long.valueOf(sourceDbIds[i]), "");
                sourceMap.put(sourceDbName[i], sourceTable.getData());
            }
            JsonResult targetTable = querygSourceTableList(Long.valueOf(targetDbId), "");
            targetMap.put(targetName, targetTable.getData());

        }
        returnMap.put("task", dmpDevelopTask);
        returnMap.put("sourceData", sourceMap);
        returnMap.put("targetData", targetMap);
        return JsonResult.ok(returnMap);
    }

}
